<?php

namespace magazine;

class Plugin_Magazine {

	private $options;
	private $path;
	private $pdf_path;
	private $thumb_path;
	private $files;
	private $structure;

	public function __construct($options) {

		$this->options = $options;
		$this->files = [];
	}


	// load pdf files from path
	public function load($path) {

		$this->path = $path;
		$this->pdf_path = $this->options['magazine_pdf'] . $path . '/';
		$this->thumb_path = $this->options['magazine_thumb'] . $path . '/';

		$files = scandir($this->pdf_path);

		foreach ($files as $file) {

			// is pdf file
			if (strtolower(pathinfo($file, PATHINFO_EXTENSION)) == 'pdf') {
				array_push($this->files, $file);

				$this->parse_file($file);
			}
		}

	}


	private function parse_file($file) {

		preg_match($this->options['rule'], $file, $val_matches);

		if (count($val_matches) > 1) {

			$text = $this->parse_format($this->options['format'], $val_matches, $options);
			$this->add($file, $text, $options);

		}

		$this->sort($this->options['sort']);
	}


	// parse format an insert data for placeholders
	private function parse_format($format, $data, &$options=[]) {

		preg_match('$(\{[0-9]+\})$', $format, $form_match, PREG_OFFSET_CAPTURE);

		// placeholder found
		if (count($form_match) && count($form_match[0]) > 1) {

			$idx = str_replace(["{","}"], "", $form_match[0][0]);

			// data for placeholder id found
			if (isset($data[$idx])) {

				// get structured data
				if ($this->options['year'] == $idx) $options['year'] = $data[$idx];
				if ($this->options['month'] == $idx) $options['month'] = $data[$idx];
				if ($this->options['issue'] == $idx) $options['issue'] = $data[$idx];

				// replace placeholder with data
				$format = str_replace($form_match[0][0], $data[$idx], $format);

				// call recursion
				$format = $this->parse_format($format, $data, $options);
			}
		}

		return $format;
	}


	// add to structure
	private function add($file, $text, $options) {

		if (isset($options['year'])) {
			$year = $options['year'];
		}
		else {
			$year = 'empty';
		}

		// create year if not exists
		if (!isset($this->structure[$year])) {
			$this->structure[$year] = [];
		}



		if (isset($options['month'])) {
			$month = $options['month'];
		}
		else {
			$month = 'empty';
		}

		// create year if not exists
		if (!isset($this->structure[$year][$month])) {
			$this->structure[$year][$month] = [];
		}


		if (isset($options['issue'])) {
			$issue = $options['issue'];
		}
		else {
			$issue = 'empty';
		}

		// create year if not exists
		if (!isset($this->structure[$year][$month][$issue])) {
			$this->structure[$year][$month][$issue] = ['file' => $file, 'text' => $text];
		}
	}


	// sort entries ASC/DESC
	private function sort($sort) {

		switch (strtolower($sort)) {

			case "asc":
				arsort($this->structure);
				break;

			case "desc":
				krsort($this->structure);
				break;
		}
	}


	// reset thumbnails
	public function reset() {

		$thumb_path = $this->options['magazine_pdf'] . 'thumb/';

		$files = scandir($thumb_path);
		foreach ($files as $file) {

		}


		return false;
	}



	// render structure
	public function render() {

		$o = '<div class="magazine_list">';

		foreach ($this->structure as $year => $months) {

			if ($year != 'empty') {
				$o .= '<div class="magazine_year">';
				$o .= '<div class="magazine_title">' . $year . '</div>';
			}

			foreach ($months as $month => $issues) {

				if ($month != 'empty') {
					$o .= '<div class="magazine_month">' . $month;
				}

				foreach ($issues as $issue) {

					$o .= '<div class="magazine_issue">';
					$o .= '<a href="' . $this->pdf_path . $issue['file'] . '">';
					$o .= $issue['text'] . "<br>";


					// show 1st page of pdf
					// $o .= $issue['file'];

//TODO
					// create image from pdf
					if (class_exists('Imagick')) {
						$o .= $this->get_from_pdf($issue['file']);
					}

					// load image
					else {
						$o .= $this->get_thumb($issue['file']);
					}

					$o .= '</a></div>';
				}

				if ($month != 'empty') {
					$o .= '</div>';
				}
			}

			if ($year != 'empty') {
				$o .= '</div>';
			}
		}

		$o .= '</div>';

		return $o;
	}


	private function get_from_pdf($path) {

		// create thumb directory if not exists
		if (!file_exists($this->pdf_path . 'thumb/')) {
			if (!mkdir ($this->pdf_path . 'thumb/')) {
				// $o = 'ERROR, can´t create thumb directory';
			}

		}

		else {
			
			$thumb_path = $this->pdf_path . 'thumb/' . pathinfo($path, PATHINFO_FILENAME) . '.jpg';

			// create thumbnail if don't exist
	// TODO clear thumbnails in admin
			if (!file_exists($thumb_path)) {
		
				$im = new \Imagick(); 
				$im->setResolution(72, 72);
				$im->readimage($this->pdf_path . $path . '[0]');
				$im->setImageFormat('jpeg'); 
				$im->writeImage($thumb_path);
				$im->clear(); 
				$im->destroy();
		
			}

			$o = '<img class="magazine_thumb" src="' . $thumb_path . '">';
		}

		return $o;
	}


	private function get_thumb($path) {

		return "";
	}

}
?>